"""Server product module."""
__doc__ = """Server product module.

$Id: Server.py,v 1.12 2000/08/28 11:02:36 heine Exp $"""

__version__='$Revision: 1.12 $'[11:-2]

from Globals import HTMLFile		# fakes a method from a DTML file
from Globals import MessageDialog
from Globals import Persistent		# makes an object stick in the ZODB
from DateTime import DateTime
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
from __main__ import *
import Zope
import string, xmlrpclib, zlib, base64, twofish, md5
from StringIO import StringIO
from types import StringType

manage_addServerForm = HTMLFile('serverAdd', globals()) 

def manage_addServer(self, title='', password='', REQUEST=None):
	"""Add a Server to a folder."""
	id='Server'
	ob=Server(id, title, password)
	self._setObject(id, ob)
	if REQUEST is not None:
		return MessageDialog(
			title = 'Added',
			message = "The Server-Module was added successfully!",
			action = 'manage_main',
			)
	
class Server(
		OFS.SimpleItem.Item,			# A simple Principia object. Not Folderish. 
		Persistent,						# Make us persistent. Yaah! 
		Acquisition.Implicit,			# Let us use our parent's variables and methods. 
		AccessControl.Role.RoleManager	# Security manager. 
	): 
		
	meta_type = 'Server'		# what do people think they're adding? 
	_uploaders=['ExtFile', 'ExtImage', 'PresseImage', 'PresseText']
	
	manage_options = ( # what management options are there?
		{'label': 'Edit Server','action': 'manage_main'		},
		{'label': 'Security',	'action': 'manage_access'	},
	) 

	__ac_permissions__=( # what permissions make sense for us? 
		('View management screens',	('manage_tabs','manage_main',)	),
		('Change permissions',		('manage_access',)				),
		('Edit Server', ('manage_editServer','manage_editClients','manage_deleteClients',)),
	)

	def __init__(self, id, title='', password=''):
		"""initialize a new instance of Server"""
		self.id = id
		self.title=title
		self.password=password
		self.clients={}
	
	manage_main = HTMLFile('serverEdit', globals())			# Management Interface
	
	def manage_editServer(self, title='', password='', REQUEST=None):
		"""does this really need a doc string?"""
		self.id = id
		self.title=title
		self.password=password
		if REQUEST is not None:
			return MessageDialog(
				title = 'Edited',
				message = "The properties of the Server have been changed!",
				action = './manage_main',
			)
	
	def manage_editClients(self, uid, url, passw, path, encrypt='Yes', REQUEST=None):
		"""manage the registered Clients"""
		if REQUEST.has_key('delete'):
			sel=REQUEST['delete']
			if (type(sel) is StringType) and self.clients.has_key(sel):
				if not uid: uid=sel
				if not url: url=self.clients[sel]['url']
				if not passw: passw=self.clients[sel]['password']
				if not path: path=self.clients[sel]['path']
				del self.clients[sel]
		if uid!='':
			self.clients[uid]={'url':url, 'password':passw, 'path':path, 'encrypt':encrypt}
			self.clients=self.clients
		if REQUEST is not None:
			return self.manage_main(self, REQUEST, update_menu=1)
	
	def manage_deleteClients(self, REQUEST):
		"""delete client(s)"""
		if REQUEST.has_key('delete'):		# Mind. eine Checkbox aktiviert
			deletes=REQUEST['delete'] 	# Mehrere Checkboxen
			if type(deletes) is StringType:
				deletes=[deletes]		# genau eine Checkbox
			for dels in deletes:
				if self.clients.has_key(dels):
					del self.clients[dels]
			self.clients=self.clients
		if REQUEST is not None:
			return self.manage_main(self, REQUEST, update_menu=1)
	
	def checkObject(self, client, crypt):
		"""check if all objects along the path are present"""
		ret=[]
		if not self.clients.has_key(client): return '401'
		crypt=base64.decodestring(crypt)				# Base64 decode
		cipher=twofish.twofish()
		cipher.hashkey(self.clients[client]['password'])
		try: crypt=cipher.decrypt(crypt)				#Twofish decrypt
		except: return '500'
		try: path=eval(crypt)
		except: return '401'
		parent=self.aq_parent
		if self.clients[client]['path']!='': 
			if hasattr(parent, 'aq_base'): parent=parent.aq_base
			try: parent=parent._getOb(self.clients[client]['path'])
			except: return '404'
		for item in path:
			try: 
				parent=parent._getOb(item)
				ret.append(item)
			except: 
				break
		crypt=`ret`
		crypt=crypt+(16-len(crypt)%16)*' '
		crypt=cipher.encrypt(crypt)
		crypt=base64.encodestring(crypt)
		return crypt
		
	def addObject(self, client, crypt, pack, backup=0, multiple=1, efupload=1):
		"""add a new object.
		   the object itself is included in 'pack' (base64 encoded and zipped),
		   the username, the path to the object and a md5-key of the object
		   are included in 'crypt' (base64 encoded, blowfish encrypted)
		   the other parameters are: 
		   backup - make backup of existing object (=1) or not (=0)
		   multiple - only one single object is transmitted (=0) or a complete object-tree (=1)
		   efupload - upload ExtFiles: =0 Never, =1 When necessary, =2 always
		"""
		
	# Part 1: decode, decompress, decrypt, extract the necessary information and check validity
		if not self.clients.has_key(client): return 'Bad Authentication.'
		crypt=base64.decodestring(crypt)				# Base64 decode
		pack=base64.decodestring(pack)
		cipher=twofish.twofish()
		cipher.hashkey(self.clients[client]['password'])
		try: crypt=cipher.decrypt(crypt)				#Twofish decrypt
		except: return 'Error decrypting object.'
		p1=string.rfind(crypt,"['")						# extract user - path - rest
		p2=string.rfind(crypt,"']")+2
		key=crypt[:p1]
		path=crypt[p1:p2]
		user=string.strip(crypt[p2:])
		if key!=md5.new(pack).digest(): return 'Bad Authentication.'
		try: path=eval(path)
		except: return 'Bad Authentication.'
		try: pack=zlib.decompress(pack)					# zlib decompress
		except: return 'Error decompressing object.'
		
		f=StringIO()
		f.write(pack)
		f.seek(0)
		
	# Part 2: Find the object, in which the new object(s) will be added and make backup, if necessary
		if hasattr(self, 'aq_parent'): parent=self.aq_parent
		else: return 'Client-Root Folder not found!'
		if self.clients[client]['path']!='': 
			try: parent=parent._getOb(self.clients[client]['path'])
			except: return 'Client-Root Folder not found!'
		urlpath=''
		for item in path[:-1]:
			try: parent=parent._getOb(item)
			except: return 'Path not found. Please try again on a higher level.'
			urlpath=urlpath+'/'+item
		
		if user=='Anonymous User': 
			return 'You do not have the premission to add this object'
		elif user=='superuser':
			try: userobj=parent.acl_users._super
			except: return 'Unknown Zope-user'
		else:
			if user in parent.get_valid_userids(): userobj=parent.acl_users.getUser(user)
			else: return 'Unknown Zope-user'
		if not (userobj.has_permission('Import/Export objects',parent)
				and userobj.has_permission('Delete objects',parent)):
			return 'You do not have the premission to add this object'
		self.REQUEST['AUTHENTICATED_USER']=userobj
		get_transaction().setUser(userobj.getUserName(),self.REQUEST.get('AUTHENTICATION_PATH'))
		
		newid=path[-1]
		try: 
			parent._getOb(newid)
		except:
			old=None
		else:
			old=parent._getOb(newid)
			parent._delObject(newid)
			if backup==1:
				count=0
				bakid='%s_backup_%i'% (newid, count)
				while hasattr(parent, bakid):
					count=count+1
					bakid='%s_backup_%i'% (newid, count)
				old._setId(bakid)
				parent._setObject(bakid, old)
		
	# Part 3: Import new object, upload ExtFiles and manage backup
		try: new=parent._p_jar.importFile(f)
		except: return "importFile failed."
		if callable(new.id):nid=new.id()
		else: nid=new.id
		if newid!=nid: return "Invalid object sent."
		new.manage_afterClone(new)
		try: parent._setObject(newid, new)
		except: return "Fatal error: setObject failed!"
		ret=''
		if efupload:
			fails=self._check_extFiles(new, client,urlpath, efupload)
			if fails!='': ret=ret+"Upload of external files failed in:<br>%s<br>" % fails
		if old is not None and multiple==0 and hasattr(old, '_objects'):
			if hasattr(new,'aq_base'): new=new.aq_base
			if hasattr(old,'aq_base'): old=old.aq_base
			for item in old._objects:
				obj=old._getOb(item['id'])
				if backup!=0:
					obj=obj._getCopy(new)
					obj.manage_afterClone(obj)
				new._setObject(item['id'], obj)
		if ret!='': return ret
		return 'ok'

	def _check_extFiles(self, object, client, path, efupload):
		ret=''
		if callable(object.id): path=path+'/'+object.id()
		else: path=path+'/'+object.id
		if self._uploaders.count(object.meta_type):
			oldpath=object.absolute_url()
			url='http://'+self.clients[client]['url']+path
			if efupload==2 or (not os.path.isfile(os.path.join(object._reposit_path(),object.filename))):
				try: object.manage_http_upload(url)
				except: ret=ret+object.id+' - '
		if hasattr(object,'_objects'):
			for d in object._objects:
				obj=object._getOb(d['id'])
				ret=ret+self._check_extFiles(obj, client, path, efupload)
		return ret